
/*
Array Methods
=================================================
 */


/*
Get the last element.

__Example:__

``` coffee
util = require 'alinex-util'
test = [ 1,2,3,4,5 ]
result = util.array.last test
```

This results to:

``` coffee
result = 5
```

@param {Array} array to read from
@param {Integer} back offset character position from the end to look for
@return last element value
 */

(function() {
  var dynamicMultiSort, dynamicSort;

  exports.last = function(array, back) {
    return array[array.length - (back || 0) - 1];
  };


  /*
  Remove duplicate entries from array.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = [ 1,2,2,3,4,1,5 ]
  result = util.array.unique test
  ```
  
  This results to:
  
  ``` coffee
  result = [1,2,3,4,5]
  ```
  
  @param {Array} array to to be checked
  @return {Array} new array with duplicates removed
   */

  exports.unique = function(array) {
    var _, j, key, output, ref, results, value;
    output = {};
    for (key = j = 0, ref = array.length; 0 <= ref ? j < ref : j > ref; key = 0 <= ref ? ++j : --j) {
      output[JSON.stringify(array[key])] = array[key];
    }
    results = [];
    for (_ in output) {
      value = output[_];
      results.push(value);
    }
    return results;
  };


  /*
  Sort array of objects.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = [
    {first: 'Johann Sebastion', last: 'Bach'}
    {first: 'Wolfgang Amadeus', last: 'Mozart'}
    {first: 'Joseph', last: 'Haydn'}
    {first: 'Richard', last: 'Wagner'}
    {first: 'Antonio', last: 'Vivaldi'}
    {first: 'Michael', last: 'Haydn'}
    {first: 'Franz', last: 'Schubert'}
  ]
   * sort by last name ascending and first name descending
  result = util.array.sortBy test, 'last', '-first'
  ```
  
  This results to:
  
  ``` coffee
  result = [
    {first: 'Johann Sebastion', last: 'Bach'}
    {first: 'Michael', last: 'Haydn'}
    {first: 'Joseph', last: 'Haydn'}
    {first: 'Wolfgang Amadeus', last: 'Mozart'}
    {first: 'Franz', last: 'Schubert'}
    {first: 'Antonio', last: 'Vivaldi'}
    {first: 'Richard', last: 'Wagner'}
  ]
  ```
  
  Like displayed above you may give one or multiple field names to sort by the
  earlier has precedence. If field name starts with '-' sign it will sort in
  descending order.
  
  @param {Array<Object>} array to to be checked
  @param {String} sort... sort column name (if prepended with '-' it will sort in
  descending order)
  @return {Array<Object>} new array in sorted order
  @throw {Error} Missing sort column name.
   */

  exports.sortBy = function(array) {
    var args;
    args = [].slice.call(arguments);
    switch (args.length) {
      case 1:
        throw new Error("Missing sort column name.");
        break;
      case 2:
        return array.sort(dynamicSort(args[1]));
      default:
        return array.sort(dynamicMultiSort.apply(this, args.slice(1)));
    }
  };


  /*
  Shuffle elements. But this only works if the array has more than two elements within
  else it will be kept unchanged.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  util.array.shuffle [1..9]
   * => [ 3, 1, 5, 6, 4, 8, 2, 9, 7 ]
  ```
  
  @param {Array} array to to be shuffled
  @return {Array} the same array
   */

  exports.shuffle = function(source) {
    var i, j, r, ref, ref1;
    if (!(source.length >= 2)) {
      return source;
    }
    for (i = j = ref = source.length - 1; ref <= 1 ? j <= 1 : j >= 1; i = ref <= 1 ? ++j : --j) {
      r = Math.floor(Math.random() * (i + 1));
      ref1 = [source[r], source[i]], source[i] = ref1[0], source[r] = ref1[1];
    }
    return source;
  };

  dynamicSort = function(field) {
    var sortOrder;
    sortOrder = 1;
    if (field[0] === '-') {
      sortOrder = -1;
      field = field.substr(1);
    }
    return function(a, b) {
      var result;
      result = a[field] < b[field] ? -1 : a[field] > b[field] ? 1 : 0;
      return result * sortOrder;
    };
  };

  dynamicMultiSort = function() {
    var args, fields;
    args = [].slice.call(arguments);
    fields = args.map(function(e) {
      return dynamicSort(e);
    });
    return function(a, b) {
      var j, len, result, sort;
      result = 0;
      for (j = 0, len = fields.length; j < len; j++) {
        sort = fields[j];
        result = sort(a, b);
        if (result) {
          return result;
        }
      }
      return result;
    };
  };

}).call(this);

//# sourceMappingURL=array.map