
/*
Clone Method
=================================================
This method will create a clone of the given object.
As possible all methods will be cloned, but if object instances are used they
will be referenced.

__Example:__

``` coffee
util = require 'alinex-util'
test = { eins: 1 }
result = util.clone test
```

This results to:

``` coffee
result = { eins: 1 }
result is test = false
```
 */

(function() {
  var chalk, debug, util;

  debug = require('debug')('util:clone');

  util = require('util');

  chalk = require('chalk');


  /*
  @name clone()
  @param {Object} obj the object to be cloned
  @param {Integer} [depth] maximum level of cloning deper levels will be referenced
  @return {Object} the cloned object
   */

  module.exports = function(obj, depth) {
    var cloned, doClone, indent;
    indent = '';
    cloned = [];
    doClone = function(obj, depth) {
      var e, flags, i, j, k, l, len, len1, len2, n, ref, ref1, res, type, v;
      indent += '   ';
      if (debug.enabled) {
        debug(chalk.grey(indent.slice(3) + "-> " + (util.inspect(obj))));
      }
      if (!obj) {
        indent = indent.slice(3);
        if (debug.enabled) {
          debug(chalk.grey(indent + "<- is undefined"));
        }
        return obj;
      }
      if ((depth != null) && depth === 0) {
        indent = indent.slice(3);
        if (debug.enabled) {
          debug(chalk.grey(indent + "<- max depth reached"));
        }
        return obj;
      }
      if ((ref = typeof obj) === 'number' || ref === 'string' || ref === 'boolean') {
        indent = indent.slice(3);
        if (debug.enabled) {
          debug(chalk.grey(indent + "<- is primitive"));
        }
        return obj;
      }
      ref1 = [Number, String, Boolean, Date];
      for (j = 0, len = ref1.length; j < len; j++) {
        type = ref1[j];
        if (obj instanceof type) {
          indent = indent.slice(3);
          if (debug.enabled) {
            debug(chalk.grey(indent + "<- is " + type));
          }
          return new type(obj);
        }
      }
      if (obj instanceof RegExp) {
        indent = indent.slice(3);
        debug(chalk.grey(indent + "<- is RegExp"));
        flags = '';
        if (obj.global) {
          flags += 'g';
        }
        if (obj.ignoreCase) {
          flags += 'i';
        }
        if (obj.multiline) {
          flags += 'm';
        }
        return new RegExp(obj.source, flags);
      }
      if (debug.enabled) {
        debug(chalk.grey(indent + "++", util.inspect(cloned)));
      }
      for (k = 0, len1 = cloned.length; k < len1; k++) {
        e = cloned[k];
        if (e[0] === obj) {
          if (debug.enabled) {
            debug(chalk.grey(indent + "<- already cloned object..."));
          }
          return e[1];
        }
      }
      if (Array.isArray(obj)) {
        res = [];
        cloned.push([obj, res]);
        for (i = l = 0, len2 = obj.length; l < len2; i = ++l) {
          n = obj[i];
          res[i] = doClone(n, depth - 1);
        }
        indent = indent.slice(3);
        if (debug.enabled) {
          debug(chalk.grey(indent + "<- cloned array..."));
        }
        return res;
      }
      if (obj.nodeType && typeof obj.cloneNode === 'function') {
        indent = indent.slice(3);
        if (debug.enabled) {
          debug(chalk.grey(indent + "<- cloned DOM node..."));
        }
        res = obj.cloneNode(true);
        cloned.push([obj, res]);
        return res;
      }
      if (obj.constructor.name !== Object.name) {
        indent = indent.slice(3);
        if (debug.enabled) {
          debug(chalk.grey(indent + "<- keep class object"));
        }
        return obj;
      }
      res = {};
      cloned.push([obj, res]);
      for (i in obj) {
        v = obj[i];
        res[i] = doClone(v, depth - 1);
      }
      indent = indent.slice(3);
      if (debug.enabled) {
        debug(chalk.grey(indent + "<- cloned object"));
      }
      return res;
    };
    return doClone(obj, depth);
  };


  /*
  Debugging
  --------------------------------------------------
  Debugging is possible using environment setting:
  
  ```
  DEBUG=util:clone    -> shows each level of cloning
  ```
  
      util:clone -> { maxCpu: '95%', maxLoad: '400%' } +0ms
      util:clone    ++ [] +0ms
      util:clone    -> '95%' +0ms
      util:clone    <- is primitive +0ms
      util:clone    -> '400%' +0ms
      util:clone    <- is primitive +0ms
      util:clone <- cloned object +0ms
      util:clone -> { nice: -20 } +0ms
      util:clone    ++ [] +0ms
      util:clone    -> -20 +0ms
      util:clone    <- is primitive +0ms
      util:clone <- cloned object +0ms
   */

}).call(this);

//# sourceMappingURL=clone.map