
/*
Extend Object
=================================================
Extend an object with another one.

This method will extend a given object with the entries from additional
objects. Therefore it will do a deep extend.

__Example:__

``` coffee
util = require 'alinex-util'
test = { eins: 1 }
util.extend test, { zwei: 2 }, { eins: 'eins' }, { drei: 3 }
```

This results to:

``` coffee
test = { zwei: 2, eins: 'eins', drei: 3 }
```

But keep in mind that this will change the first object, to the result, too.
 */

(function() {
  var chalk, clone, debug, extend, indent, util,
    slice = [].slice,
    indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
    hasProp = {}.hasOwnProperty;

  debug = require('debug')('util:extend');

  util = require('util');

  chalk = require('chalk');

  clone = require('./clone');

  indent = '';


  /*
  @param {String} [mode] containing one or multiple keys which have to be separated
  by space or comma:
  - `CLONE` - clone object and all extenders before extending, keeps the resulting
    objects untouched (works only globally)
  - `OVERWRITE` - allow overwrite of array and object mode in specific elements
  - `ARRAY_CONCAT` - (default) if no other array-mode set, will concat additional
    elements
  - `ARRAY_REPLACE` - for all arrays, replace the previouse array completely instead
    of extending them
  - `ARRAY_OVERWRITE` - overwrite the same index instead of extending the array
  - `OBJECT_EXTEND` - (default) if no other object-mode given, will add/replace
    properties with the new ones
  - `OBJECT_REPLACE` - will always replace the object completely with the new one,
    if the keys are different
  @param {Object} obj object to be extended
  @param {Object} [ext...] extenders to be applied to the base one
  @return {Object} new combined object
  @description
  The `mode` may also be changed on any specific element by giving a different mode
  just for this operation in the extending element itself. Therefore an array
  should has the mode as first element or an object as an attribute.
   */

  module.exports = extend = function() {
    var cmode, ext, i, j, k, l, len, len1, len2, m, mode, n, obj, ref, ref1, src, v;
    ext = 1 <= arguments.length ? slice.call(arguments, 0) : [];
    indent += '   ';
    mode = [];
    if (typeof ext[0] === 'string' && ((ref = ext[0]) != null ? ref.indexOf('MODE' === 0) : void 0)) {
      mode = ext.shift().split(' ').slice(1);
    }
    if (!ext.length) {
      return null;
    }
    if (indexOf.call(mode, 'CLONE') >= 0) {
      ext = clone(ext);
    }
    mode = mode.filter(function(e) {
      return e !== 'CLONE';
    });
    obj = ext.shift();
    if (debug.enabled) {
      debug(indent.slice(3) + "-> extend " + (chalk.grey(util.inspect(obj))));
      if (mode.length) {
        debug(indent.slice(3) + "   using mode: " + (mode.join(', ')));
      }
    }
    for (j = 0, len = ext.length; j < len; j++) {
      src = ext[j];
      if (src == null) {
        continue;
      }
      if (((ref1 = src.constructor) != null ? ref1.name : void 0) === Object.name && !Object.keys(src).length) {
        continue;
      }
      if (debug.enabled) {
        debug(indent.slice(3) + "   by " + (chalk.grey(util.inspect(src))));
      }
      if (obj == null) {
        obj = src;
        continue;
      }
      if (Array.isArray(src)) {
        cmode = mode;
        if (indexOf.call(mode, 'OVERWRITE') >= 0 && typeof src[0] === 'string' && src[0].slice(0, 5) === 'MODE ') {
          cmode = src.shift().split(' ').slice(1);
          if (debug.enabled) {
            debug(indent.slice(3) + "   change mode: " + (cmode.join(', ')));
          }
        } else {
          mode;
        }
        if (indexOf.call(cmode, 'ARRAY_REPLACE') >= 0 || !Array.isArray(obj)) {
          obj = src;
          continue;
        }
        if (indexOf.call(cmode, 'ARRAY_OVERWRITE') >= 0) {
          for (i = l = 0, len1 = src.length; l < len1; i = ++l) {
            n = src[i];
            obj[i] = extend("MODE " + (mode.join(' ')), obj[i], n);
          }
          continue;
        }
        for (m = 0, len2 = src.length; m < len2; m++) {
          n = src[m];
          obj.push(n);
        }
        continue;
      }
      if (typeof src !== 'object') {
        obj = src;
        continue;
      }
      cmode = mode;
      if (indexOf.call(mode, 'OVERWRITE') >= 0 && src.OBJECT_REPLACE) {
        cmode = ['OBJECT_REPLACE'];
        delete src.OBJECT_REPLACE;
        if (debug.enabled) {
          debug(indent.slice(3) + "   change mode: " + (cmode.join(', ')));
        }
      }
      if (!(obj != null) || Array.isArray(obj) || typeof obj !== 'object' || obj.constructor.name !== Object.name) {
        obj = src;
        continue;
      }
      if (indexOf.call(cmode, 'OBJECT_REPLACE') >= 0 && Object.keys(obj).join(',') !== Object.keys(src).join(',')) {
        obj = src;
        continue;
      }
      for (k in src) {
        if (!hasProp.call(src, k)) continue;
        v = src[k];
        if (v === null) {
          delete obj[k];
          continue;
        }
        obj[k] = indexOf.call(Object.keys(obj), k) >= 0 ? mode.length ? extend("MODE " + (mode.join(' ')), obj[k], v) : extend(obj[k], v) : v;
      }
    }
    indent = indent.slice(3);
    if (debug.enabled) {
      debug(indent + "<- " + (chalk.grey(util.inspect(obj))));
    }
    return obj;
  };


  /*
  Mode Settings
  -----------------------------------------------------------
  You may set a mode globally or in a specific level like described in the method
  definition.
  
  See the example below which replaces arrays instead of appending to them (the default).
  
  ``` coffee
  test1 = {a: [1, 2, 3], b: [1, 2, 3], c: [1, 2, 3]}
  test2 = {a: [4, 5, 6], c: ['a']}
  ext = util.extend 'MODE ARRAY_REPLACE', test1, test2
   * ext = {a: [4, 5, 6], b: [1, 2, 3], c: ['a']}
  ```
  
  And you may also change the mode only for one element addition (here the first array):
  
  ``` coffee
  test1 = {a: [1, 2, 3], b: [1, 2, 3], c: [1, 2, 3]}
  test2 = {a: ['MODE ARRAY_REPLACE', 4, 5, 6], c: ['a']}
  ext = util.extend test1, test2
   * ext = {a: [4, 5, 6], b: [1, 2, 3], c: [1, 2, 3, 'a']}
  ```
  
  And to set a mode in an object you give it as argument with value `true`:
  
  ``` coffee
  test1 = {t1: {a: 1, b: 2, c: 3}, t2: {d: 4, e: 5, f: 6}}
  test2 = {t1: {OBJECT_REPLACE: true, a: 4, b: 5}, t2: {d: 9}}
  ext = util.extend test1, test2
   * ext = {t1: {a: 4, b: 5}, t2: {d: 9, e: 5, f: 6}}
  ```
  
  
  Debugging
  --------------------------------------------------
  Debugging is possible using environment setting:
  
  ```
  DEBUG=util:extend    -> shows each level of cloning
  ```
  
      util:extend -> extend { eins: 1 } +0ms
      util:extend    by { zwei: 2 } +1ms
      util:extend    by { eins: 'eins' } +0ms
      util:extend    -> extend 1 +0ms
      util:extend       by 'eins' +0ms
      util:extend    <- 'eins' +0ms
      util:extend    by { drei: 3 } +0ms
      util:extend <- { eins: 'eins', zwei: 2, drei: 3 } +0ms
   */

}).call(this);

//# sourceMappingURL=extend.map