
/*
Number Methods
=================================================
 */


/*
This method will check that the given value is an integer.

@param value the number to check
@return {Boolean} `true` if `value` is an integer
 */

(function() {
  var parseInteger;

  exports.isInteger = function(value) {
    return value === (value | 0);
  };


  /*
  This method will check that the given value is a number.
  
  @param value the number to check
  @return {Boolean} `true` if `value` is a number
   */

  exports.isNumber = function(value) {
    return !(isNaN(parseFloat(value))) && isFinite(value);
  };


  /*
  Strict parsing of numbers.
  This will fail on all strings which are no real integer.
  
  @param value the number to check
  @return {Number} value as Number or `NaN`
  @see To check if you got a number or invalid string use {@link isNaN()}.
   */

  exports.parseInt = parseInteger = function(value) {
    if (/^(\-|\+)?([0-9]+|Infinity)$/.test(value)) {
      return Number(value);
    }
    return 0/0;
  };


  /*
  Parse human readable time to seconds.
  With this parser it is possible to read a time interval in human format
  like: `1h 30m`.
  
  __Example:__
  
  ``` coffee
  number.parseSeconds 200          # 200
  number.parseSeconds '200s'       # 200
  number.parseSeconds '200S'       # 200
  number.parseSeconds '5m'         # 300
  number.parseSeconds '1h'         # 3600
  number.parseSeconds '2.5h'       # 9000
  number.parseSeconds '1d'         # 86400
  number.parseSeconds '2h 5m 100s' # 7600
  ```
  
  @param {String} value the time to transform into seconds
  @return {Number} value as Number or `NaN`
  @see {@link parseMSeconds}
  @description
  To check if you got a number or invalid string use `isNaN()`.
   */

  exports.parseSeconds = function(value) {
    var i, int, len, match, part, ref;
    int = parseInteger(value);
    if (!isNaN(int)) {
      return int;
    }
    if (typeof value !== 'string') {
      return 0/0;
    }
    int = 0;
    ref = value.toLowerCase().replace(/([smhd])(\d)/g, '$1 $2').split(/\s+/);
    for (i = 0, len = ref.length; i < len; i++) {
      part = ref[i];
      match = /^([+-]?\d+(?:\.\d+)?)\s*([smhd])$/.exec(part);
      if (!match) {
        return 0/0;
      }
      value = parseFloat(match[1]);
      int += (function() {
        switch (match[2]) {
          case 's':
            return value;
          case 'm':
            return value * 60;
          case 'h':
            return value * 3600;
          case 'd':
            return value * 24 * 3600;
        }
      })();
    }
    return Math.floor(int);
  };


  /*
  Parse human readable time to milliseconds. It will work the same as above and
  you may specify `ms` as additional unit here.
  
  @param {String} value the time to transform into seconds
  @return {Number} value as Number or `NaN`
  @see {@link parseSeconds}
  @description
  To check if you got a number or invalid string use `isNaN()`.
   */

  exports.parseMSeconds = function(value) {
    var i, int, len, match, part, ref;
    int = parseInteger(value);
    if (!isNaN(int)) {
      return int;
    }
    if (typeof value !== 'string') {
      return 0/0;
    }
    int = 0;
    ref = value.toLowerCase().replace(/([smhd])(\d)/g, '$1 $2').split(/\s+/);
    for (i = 0, len = ref.length; i < len; i++) {
      part = ref[i];
      match = /^([+-]?\d+(?:\.\d+)?)\s*([smhd]|ms)$/.exec(part);
      if (!match) {
        return 0/0;
      }
      value = parseFloat(match[1]);
      int += (function() {
        switch (match[2]) {
          case 'ms':
            return value;
          case 's':
            return value * 1000;
          case 'm':
            return value * 60000;
          case 'h':
            return value * 3600000;
          case 'd':
            return value * 24 * 3600000;
        }
      })();
    }
    return Math.floor(int);
  };

}).call(this);

//# sourceMappingURL=number.map