
/*
Object Methods
=================================================
 */

(function() {
  var debug, debugPathSearch, lcKeys, pathSearch, util,
    indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  debug = require('debug')('util:object');

  debugPathSearch = require('debug')('util:object:pathsearch');

  util = require('util');


  /*
  This method will check if an object is empty. This is also true for undefined
  objects.
  
  @param {Object} object to be checked
  @return {Boolean} `true` if object is empty
   */

  module.exports.isEmpty = function(obj) {
    if (obj == null) {
      return true;
    }
    if (obj.length != null) {
      return obj.length === 0;
    }
    return Object.keys(obj).length === 0;
  };


  /*
  This method allows you to access an element deep in an object's structure by
  giving only the path to the element.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test =
    string: 'test'
    object:
      numbers:
        one: 1
        two: 2
    list: [
      one: 11
    ,
      two: 12
    ]
  result = util.object.path test, '/object/numbers'
  ```
  
  This results to:
  
  ``` coffee
  result = { one: 1, two: 2 }
  ```
  
  @param {Object} object to be searched
  @param {String|Array} path specifying which element to reference
  @param {String|RegExp} [separator=/] used as separator
  @return element at the position of the path or `undefined` if not found
   */

  exports.path = function(obj, path, separator) {
    var i, k, len, ref;
    if (separator == null) {
      separator = '/';
    }
    if (typeof path === 'string') {
      path = path.split(separator);
    }
    if (debug.enabled) {
      debug("get path " + (util.inspect(path)) + " from " + (util.inspect(obj)));
    }
    ref = obj;
    for (i = 0, len = path.length; i < len; i++) {
      k = path[i];
      if (!k) {
        continue;
      }
      ref = ref[k];
    }
    return ref;
  };

  pathSearch = exports.pathSearch = function(obj, path, separator) {
    var cur, key, result, val;
    if (separator == null) {
      separator = '/';
    }
    if (typeof path === 'string') {
      path = path.split(separator);
    }
    if (debugPathSearch.enabled) {
      debugPathSearch("get path " + (util.inspect(path)) + " from " + (util.inspect(obj)));
    }
    if (!path.length) {
      return obj;
    }
    cur = path.shift();
    while (cur === '' && path.length) {
      cur = path.shift();
    }
    if (cur === '') {
      return obj;
    }
    result = obj;
    switch (false) {
      case cur !== '*':
        if (!path.length) {
          return obj;
        }
        if (!path.length) {
          result = [];
          for (key in obj) {
            val = obj[key];
            result.push(val);
          }
          return result;
        }
        for (key in obj) {
          val = obj[key];
          if (typeof val !== 'object') {
            continue;
          }
          result = pathSearch(val, path.slice(0));
          if (result != null) {
            return result;
          }
        }
        break;
      case cur !== '**':
        if (!path.length) {
          return obj;
        }
        result = pathSearch(result, path.slice(0));
        if (result != null) {
          return result;
        }
        path.unshift(cur);
        for (key in obj) {
          val = obj[key];
          if (typeof val !== 'object') {
            continue;
          }
          result = pathSearch(val, path.slice(0));
          if (result != null) {
            return result;
          }
        }
        break;
      case !cur.match(/\W/):
        cur = new RegExp("^" + cur + "$");
        result = [];
        for (key in obj) {
          val = obj[key];
          if (key.match(cur)) {
            result.push(val);
          }
        }
        if (!result.length) {
          return;
        }
        if (result.length === 1) {
          result = result[0];
        }
        return result;
      default:
        result = obj != null ? obj[cur] : void 0;
        if (result == null) {
          return;
        }
        if (path.length) {
          return pathSearch(result, path);
        }
        return result;
    }
  };


  /*
  Like the {@link Array.filter()} method of arrays you may filter object entries here.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test =
    one: 1
    two: 2
    three: 3
    four: 4
  result = util.object.filter test, (value) -> value < 3
  ```
  
  This results to:
  
  ``` coffee
  result =
    one: 1
    two: 2
  ```
  
  @param {Object} obj to be filtered
  @param {Function} will be called for each entry with:
  - `value` - the current value
  - `key` - `String` the current key name
  - `obj` - the whole object
  
  It should return a `Boolean` value. If `true` the entry will be copied to the
  resulting object.
  @return {Object} data object with all or some entries from the original
   */

  exports.filter = function(obj, allow) {
    var key, result;
    result = {};
    for (key in obj) {
      if (obj.hasOwnProperty(key) && allow(obj[key], key, obj)) {
        result[key] = obj[key];
      }
    }
    return result;
  };


  /*
  Make the keys within a deep object all lowercase.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test =
    One: 1
    TWO:
      three: 3
      fouR: 4
  result = util.object.lcKeys test
  ```
  
  Will result in:
  
  ``` coffee
  result =
    one: 1
    two:
      three: 3
      four: 4
  ```
  
  @param {Object} obj to be optimized
  @return {Object} data object with all keys in lowercase
   */

  lcKeys = exports.lcKeys = function(obj) {
    var i, key, lc, len, ref1;
    if (typeof obj !== 'object' || Array.isArray(obj)) {
      return obj;
    }
    lc = {};
    ref1 = Object.keys(obj);
    for (i = 0, len = ref1.length; i < len; i++) {
      key = ref1[i];
      lc[key.toLowerCase()] = lcKeys(obj[key]);
    }
    return lc;
  };


  /*
  This will check an object if it contains circular references.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = { eins: 1 }
  util.object.isCyclic test # will be false
  test.zwei = test.eins
  util.object.isCyclic test # will be true
  ```
  
  @param {Object} obj to be checked
  @return {Boolean} `true` if the object contains circular references
   */

  exports.isCyclic = function(obj) {
    var checked, detect;
    checked = [];
    detect = function(obj) {
      var key;
      if (obj && typeof obj === 'object') {
        if (indexOf.call(checked, obj) >= 0) {
          return true;
        }
        checked.push(obj);
        for (key in obj) {
          if (obj.hasOwnProperty(key) && detect(obj[key])) {
            return true;
          }
        }
      }
      return false;
    };
    return detect(obj);
  };


  /*
  Detects circular references but instead of only checking it list all cyclic objects.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = { eins: 1 }
  util.object.getCyclic test # empty list []
  test.zwei = test.eins
  util.object.getCyclic test # [{ eins: 1 }]
  ```
  
  @param {Object} obj to be checked
  @return {Array} list of objects which are circular
   */

  exports.getCyclic = function(obj) {
    var checked, cyclic, detect;
    checked = [];
    cyclic = [];
    detect = function(obj) {
      var key, results;
      if (obj && typeof obj === 'object') {
        if (indexOf.call(checked, obj) >= 0 && !(indexOf.call(cyclic, obj) >= 0)) {
          return cyclic.push(obj);
        }
        checked.push(obj);
        results = [];
        for (key in obj) {
          if (obj.hasOwnProperty(key)) {
            results.push(detect(obj[key]));
          } else {
            results.push(void 0);
          }
        }
        return results;
      }
    };
    detect(obj);
    return cyclic;
  };

}).call(this);

//# sourceMappingURL=object.map