
/*
String Methods
=================================================
 */

(function() {
  var debug, util;

  debug = require('debug')('util:string');

  util = require('util');


  /*
  This is a handy method to check that a certain string starts with the given
  phrase.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = 'abcdefg'
  result = util.string.starts test, 'ab'
  ```
  
  This results to:
  
      result = true
  
  @param {String} string text to be checked
  @param {String} literal phrase to match
  @param {Integer} [start] offset character position to start looking for
  @return {Boolean} `true` if string starts with literal
   */

  exports.starts = function(string, literal, start) {
    if (debug.enabled) {
      debug("check " + (util.inspect(string)) + " to start with " + (util.inspect(literal)) + " " + (start ? 'starting at #' + start : ''));
    }
    return literal === string.substr(start, literal.length);
  };


  /*
  Peek at the end of a given string to see if it matches a sequence.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = 'abcdefg'
  result = util.string.ends test, 'fg'
  ```
  
  This results to:
  
      result = true
  
  @param {String} string text to be checked
  @param {String} literal phrase to match
  @param {Integer} [back] offset character position from the end
  @return {Boolean} `true` if string ends with literal
   */

  exports.ends = function(string, literal, back) {
    var len;
    if (debug.enabled) {
      debug("check " + (util.inspect(string)) + " to end with " + (util.inspect(literal)) + " " + (back ? 'starting before #' + back : ''));
    }
    len = literal.length;
    return literal === string.substr(string.length - len - (back || 0), len);
  };


  /*
  Repeat a given string multiple times.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = 'ab'
  result = util.string.repeat test, 3
  ```
  
  This results to:
  
      result = 'ababab'
  
  @param {String} string phrase to repeat
  @param {Integer} num number of repeats to do
  @return {String} the repeated text
   */

  exports.repeat = function(string, num) {
    var res;
    if (debug.enabled) {
      debug("repeat " + (util.inspect(string)) + " " + num + " times");
    }
    res = '';
    while (num > 0) {
      if (num & 1) {
        res += string;
      }
      num >>>= 1;
      string += string;
    }
    return res;
  };


  /*
  Left pad string to specified length.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = '5'
  result = util.string.lpad test, 3, '0'
  ```
  
  This results to:
  
      result = '005'
  
  @param {String} string text to be padded
  @param {Integer} length final length of text
  @param {Character} [char=' '] character used for padding
  @return {String} the padded text
   */

  exports.lpad = function(string, length, char) {
    if (char == null) {
      char = ' ';
    }
    if (debug.enabled) {
      debug("left padding " + (util.inspect(string)) + " with " + (util.inspect(char)) + " to length of " + length);
    }
    if (typeof string !== 'string') {
      string = string.toString();
    }
    if (!(length >= string.length)) {
      return string;
    }
    return exports.repeat(char, length - string.length) + string;
  };


  /*
  Right pad string to specified length.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = 'abc'
  result = util.string.rpad test, 5, ' '
  ```
  
  This results to:
  
      result = 'abc  '
  
  @param {String} string text to be padded
  @param {Integer} length final length of text
  @param {Character} [char=' '] character used for padding
  @return {String} the padded text
   */

  exports.rpad = function(string, length, char) {
    if (char == null) {
      char = ' ';
    }
    if (debug.enabled) {
      debug("right padding " + (util.inspect(string)) + " with " + (util.inspect(char)) + " to length of " + length);
    }
    if (typeof string !== 'string') {
      string = string.toString();
    }
    if (!(length >= string.length)) {
      return string;
    }
    return string + exports.repeat(char, length - string.length);
  };


  /*
  Center pad string to specified length. This will add the padding on both sides.
  For udd number of padding the padding on the right will be one character more as
  on the left.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = 'abc'
  result = util.string.cpad test, 5, ' '
  ```
  
  This results to:
  
      result = 'abc  '
  
  @param {String} string text to be padded
  @param {Integer} length final length of text
  @param {Character} [char=' '] character used for padding
  @return {String} the padded text
   */

  exports.cpad = function(string, length, char) {
    var lpad, rpad;
    if (char == null) {
      char = ' ';
    }
    if (debug.enabled) {
      debug("center padding " + (util.inspect(string)) + " with " + (util.inspect(char)) + " to length of " + length);
    }
    if (typeof string !== 'string') {
      string = string.toString();
    }
    if (!(length >= string.length)) {
      return string;
    }
    lpad = exports.repeat(char, Math.floor((length - string.length) / 2));
    rpad = exports.repeat(char, Math.ceil((length - string.length) / 2));
    return lpad + string + rpad;
  };


  /*
  Trim given characters.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = '/var/local/'
  result = util.string.trim test, '/'
  ```
  
  This results to:
  
      result = 'var/local'
  
  @param {String} string text to be trimmed
  @param {String} [chars=' \n\t'] list of characters to trim off
  @return {String} the trimmed text
   */

  exports.trim = function(string, chars) {
    if (chars == null) {
      chars = " \n\t";
    }
    if (debug.enabled) {
      debug("trim " + (util.inspect(chars)) + " from  " + (util.inspect(string)));
    }
    while (string && ~chars.indexOf(string.charAt(0))) {
      string = string.substring(1);
    }
    while (string && ~chars.indexOf(string.charAt(string.length - 1))) {
      string = string.substring(0, string.length - 1);
    }
    return string;
  };


  /*
  Left trim given characters.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = '/var/local/'
  result = util.string.ltrim test, '/'
  ```
  
  This results to:
  
      result = 'var/local/'
  
  @param {String} string text to be trimmed
  @param {String} [chars=' \n\t'] list of characters to trim off
  @return {String} the trimmed text
   */

  exports.ltrim = function(string, chars) {
    if (chars == null) {
      chars = " \n\t";
    }
    if (debug.enabled) {
      debug("ltrim " + (util.inspect(chars)) + " from  " + (util.inspect(string)));
    }
    while (string && ~chars.indexOf(string.charAt(0))) {
      string = string.substring(1);
    }
    return string;
  };


  /*
  Right trim given characters.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = '/var/local/'
  result = util.string.rtrim test, '/'
  ```
  
  This results to:
  
      result = '/var/local'
  
  @param {String} string text to be trimmed
  @param {String} [chars=' \n\t'] list of characters to trim off
  @return {String} the trimmed text
   */

  exports.rtrim = function(string, chars) {
    if (chars == null) {
      chars = " \n\t";
    }
    if (debug.enabled) {
      debug("rtrim " + (util.inspect(chars)) + " from  " + (util.inspect(string)));
    }
    while (string && ~chars.indexOf(string.charAt(string.length - 1))) {
      string = string.substring(0, string.length - 1);
    }
    return string;
  };


  /*
  Make first letter upper case.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = 'abcdefg'
  result = util.string.ucfirst test
  ```
  
  This results to:
  
      result = 'Abcdefg'
  
  @param {String} string text to be changed
  @return {String} the text with first letter upper case
   */

  exports.ucFirst = function(string) {
    if (debug.enabled) {
      debug("upper case first letter of " + (util.inspect(string)));
    }
    return string.charAt(0).toUpperCase() + string.slice(1);
  };


  /*
  Make first letter lower case.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = 'ABCDEFG'
  result = util.string.lcFirst test
  ```
  
  This results to:
  
      result = 'aBCDEFG'
  
  @param {String} string text to be changed
  @return {String} the text with first letter lower case
   */

  exports.lcFirst = function(string) {
    if (debug.enabled) {
      debug("lower case first letter of " + (util.inspect(string)));
    }
    return string.charAt(0).toLowerCase() + string.slice(1);
  };


  /*
  Check if string contains substring.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = 'abcdefg'
  result1 = util.string.contains test, 'bc'
  result2 = util.string.contains test, 'gh'
  ```
  
  This results to:
  
      result1 = true
      result2 = false
  
  @param {String} string text to be checked
  @param {String} phrase text to be searched for
  @return {Boolean} `true` if phrase is contained in string
   */

  exports.contains = function(string, phrase) {
    if (debug.enabled) {
      debug("check if " + (util.inspect(string)) + " contains " + (util.inspect(phrase)));
    }
    return string.indexOf(phrase) !== -1;
  };


  /*
  Word wraps a given text as needed.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = "All necessary parts are on the same machine, so that you only have to bring this
  machine to work. Backups of the data are made on vs10152.
  \n\n
  Keep in mind that the machine is in the test net and you have to use a valid VPN
  connection for accessing.
  """
  result = util.string.wordwrap test, 78
  ```
  
  This results to (variable result):
  
      All necessary parts are on the same machine, so that you only have to bring
      this machine to work. Backups of the data are made on vs10152.
  
      Keep in mind that the machine is in the test net and you have to use a valid
      VPN connection for accessing.
  
  @param {String} str
  @param {Integer} [width=80] maximum amount of characters per line
  @param {String} [brk=\n] string that will be added whenever it's needed to break the line
  @return {String} the multiline text
   */

  exports.wordwrap = function(str, width, brk) {
    var p, wrap;
    if (width == null) {
      width = 80;
    }
    if (brk == null) {
      brk = '\n';
    }
    if (debug.enabled) {
      debug("wordwrap to width of " + width + " characters (separator is " + (util.inspect(brk)) + "): \n" + str);
    }
    wrap = '';
    while (true) {
      if (str.length <= width) {
        wrap += str;
        break;
      }
      p = str.indexOf(brk);
      if ((-1 < p && p <= width)) {
        wrap += str.slice(0, +p + 1 || 9e9);
        str = str.slice(p + 1);
        continue;
      }
      p = width;
      while (p > 0 && str[p] !== ' ') {
        p--;
      }
      if (p > 0) {
        wrap += str.slice(0, +(p - 1) + 1 || 9e9) + brk;
        str = str.slice(p + 1);
        continue;
      }
      p = width;
      while (p < str.length && str[p] !== ' ') {
        p++;
      }
      if (p < str.length) {
        wrap += str.slice(0, +(p - 1) + 1 || 9e9) + brk;
        str = str.slice(p + 1);
        continue;
      }
      wrap += str;
      break;
    }
    return wrap;
  };


  /*
  This will shorten the given text and add ellipsis if it is too long. This is done
  word aware.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = """
  A high CPU usage means that the server may not start
  another task immediately.
  If the load is also very high the system is overloaded, check if any application
  goes evil.
  """
  result = util.string.shorten test, 68
  ```
  
  This results to (variable result):
  
      A high CPU usage means that the server may not start...
  
  @param {String} str text to shorten
  @param {Integer} limit maximum number of characters
  @return {String} shortend text
   */

  exports.shorten = function(str, limit) {
    if (debug.enabled) {
      debug("shorten to width of " + limit + " characters:\n" + str);
    }
    if (str.length < limit) {
      return str;
    }
    str = str.slice(0, +(limit - 3) + 1 || 9e9).replace(/\s\S*?$/, '');
    return str + '...';
  };


  /*
  Convert text into list or list of array (recordset).
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = 'one,two,three\n1,2,3\n4,5,6'
  list = util.string.toList test, /\n/
  table = util.string.toList test, /\n/, /,/
  ```
  
  This results to:
  
  ``` coffee
  list = [
    'one,teo,three'
    '1,2,3'
    '4,5,6'
  ]
  table = [
    ['one', 'two', 'three']
    ['1', '2', '3']
    ['4', '5', '6']
  ]
  ```
  
  @param {String} text data to split
  @param {String|RegExp} [rowDelimiter=\n] text or regular expression to divide into lines
  @param {String|RegExp\ [colDelimiter] text or regular expression to divide into columns
  @return {Array} the new list or list of arrays
   */

  exports.toList = function(text, rowDelimiter, colDelimiter) {
    var list;
    if (rowDelimiter == null) {
      rowDelimiter = /\n/;
    }
    if (debug.enabled) {
      debug("split to list (" + (util.inspect(rowDelimiter)) + "/" + (util.inspect(colDelimiter)) + "):\n" + text);
    }
    list = text.split(rowDelimiter);
    if (!colDelimiter) {
      return list;
    }
    return list.map(function(e) {
      return e.split(colDelimiter);
    });
  };


  /*
  Convert text into regular expression object if possible.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = '/\n/'
  test = util.string.toRegExp test
  list = util.string.split test # use the RegExp
  ```
  
  @param {String} text to be converted
  @return {RegExp|String} regular expression or ioriginal text
   */

  exports.toRegExp = function(text) {
    var match;
    if (debug.enabled) {
      debug("try to transform " + (util.inspect(text)) + " to RegExp");
    }
    if (typeof text !== 'string') {
      return text;
    }
    match = text.replace('\t', '\\t').replace('\n', '\\n').replace('\r', '\\r').match(/^\/(.*)\/([gim]+)?$/);
    if (!match) {
      return text;
    }
    return new RegExp(match[1], match[2]);
  };


  /*
  Mask special characters for html display.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = '<html>'
  test = util.string.htmlEncode test
  ```
  
  @param {String} text to be encoded
  @return {RegExp|String} encoded text
   */

  exports.htmlEncode = function(text) {
    var map;
    if (debug.enabled) {
      debug("encode " + text + " for html");
    }
    if (typeof text !== 'string') {
      return text;
    }
    map = {
      "&": "&amp;",
      "<": "&lt;",
      ">": "&gt;",
      "\"": "&quot;",
      "'": "&#39;"
    };
    return text.replace(/[&<>"']/g, function(m) {
      return map[m];
    });
  };


  /*
  Unmask special characters for html display.
  
  __Example:__
  
  ``` coffee
  util = require 'alinex-util'
  test = '&lt;html&gt;'
  test = util.string.htmlDecode test
  ```
  
  @param {String} text to be decoded
  @return {RegExp|String} decoded text
   */

  exports.htmlDecode = function(text) {
    var map;
    if (debug.enabled) {
      debug("encode " + text + " for html");
    }
    if (typeof text !== 'string') {
      return text;
    }
    map = {
      "&amp;": "&",
      "&lt;": "<",
      "&gt;": ">",
      "&quot;": "\"",
      "&#39;": "'"
    };
    return text.replace(/(&amp;|&lt;|&gt;|&quot;|&#39;)/g, function(m) {
      return map[m];
    });
  };


  /*
  Debugging
  --------------------------------------------------
  Debugging is possible using environment setting:
  
  ```
  DEBUG=util:string    -> each method call
  ```
   */

}).call(this);

//# sourceMappingURL=string.map